"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("mocha");
const chai_1 = require("chai");
const sinon_1 = require("sinon");
const local_cache_1 = require("./local-cache");
describe('it saves values in memory', () => {
    let localCache;
    before(() => {
        localCache = new local_cache_1.default();
    });
    it('has created the local cache', () => {
        chai_1.expect(localCache.isReady).to.equal(true);
    });
    it('sets a value in the cache', (done) => {
        const successCallback = sinon_1.spy();
        localCache.set('firstname', 1, 'Wolfram', successCallback);
        setTimeout(() => {
            chai_1.expect(successCallback).to.have.callCount(1);
            chai_1.expect(successCallback).to.have.been.calledWith(null);
            done();
        }, 1);
    });
    it('retrieves an existing value from the cache', (done) => {
        const successCallback = sinon_1.spy();
        localCache.get('firstname', successCallback);
        setTimeout(() => {
            chai_1.expect(successCallback).to.have.callCount(1);
            chai_1.expect(successCallback).to.have.been.calledWith(null, 1, 'Wolfram');
            done();
        }, 1);
    });
    it('deletes a value from the cache', (done) => {
        const successCallback = sinon_1.spy();
        localCache.delete('firstname', successCallback);
        setTimeout(() => {
            chai_1.expect(successCallback).to.have.callCount(1);
            chai_1.expect(successCallback).to.have.been.calledWith(null);
            done();
        }, 1);
    });
    it('tries to retrieve a non-existing value from the cache', (done) => {
        const successCallback = sinon_1.spy();
        localCache.get('firstname', successCallback);
        setTimeout(() => {
            chai_1.expect(successCallback).to.have.callCount(1);
            chai_1.expect(successCallback).to.have.been.calledWith(null, -1, null);
            done();
        }, 1);
    });
});
//# sourceMappingURL=local-cache.spec.js.map