"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("mocha");
const chai_1 = require("chai");
const sinon_1 = require("sinon");
const std_out_logger_1 = require("./std-out-logger");
const C = require("../constants");
describe('logs to stdout and stderr', () => {
    const logger = new std_out_logger_1.default({ color: false });
    const originalStdOut = process.stdout;
    const originalStdErr = process.stderr;
    const stdout = sinon_1.spy();
    const stderr = sinon_1.spy();
    const comp = function (std, exp) {
        return std.lastCall.args[0].indexOf(exp) !== -1;
    };
    before(() => {
        Object.defineProperty(process, 'stdout', {
            value: { write: stdout }
        });
        Object.defineProperty(process, 'stderr', {
            value: { write: stderr }
        });
    });
    after(() => {
        Object.defineProperty(process, 'stdout', {
            value: originalStdOut
        });
        Object.defineProperty(process, 'stderr', {
            value: originalStdErr
        });
    });
    it('creates the logger', () => {
        chai_1.expect(logger.isReady).to.equal(true);
        logger.log(C.LOG_LEVEL.INFO, C.EVENT.INFO, 'b');
        chai_1.expect(comp(stdout, 'INFO | b')).to.equal(true);
    });
    it('logs to stderr', () => {
        stdout.resetHistory();
        logger.log(C.LOG_LEVEL.ERROR, C.EVENT.INFO, 'e');
        chai_1.expect(stdout).to.have.callCount(0);
        chai_1.expect(stderr).to.have.callCount(1);
    });
    it('logs above log level', () => {
        logger.setLogLevel(C.LOG_LEVEL.DEBUG);
        stdout.resetHistory();
        logger.log(C.LOG_LEVEL.INFO, C.EVENT.INFO, 'e');
        chai_1.expect(stdout).to.have.callCount(1);
        logger.setLogLevel(C.LOG_LEVEL.WARN);
        stdout.resetHistory();
        logger.log(C.LOG_LEVEL.INFO, C.EVENT.INFO, 'e');
        chai_1.expect(stdout).to.have.callCount(0);
    });
});
//# sourceMappingURL=std-out-logger.spec.js.map