'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable class-methods-use-this */
const events_1 = require("events");
const message_parser_1 = require("../../../binary-protocol/src/message-parser");
class HTTPSocketWrapper extends events_1.EventEmitter {
    constructor(options, onMessageCallback, onErrorCallback) {
        super();
        this.onMessageCallback = onMessageCallback;
        this.onErrorCallback = onErrorCallback;
        this.uuid = Math.random();
        this.isClosed = false;
    }
    init(authResponseData, messageIndex, messageResults, responseCallback, requestTimeoutId) {
        this.user = authResponseData.userId || authResponseData.username || 'OPEN';
        this.clientData = authResponseData.clientData;
        this.authData = authResponseData.serverData;
        this.correlationIndex = messageIndex;
        this.messageResults = messageResults;
        this.responseCallback = responseCallback;
        this.requestTimeout = requestTimeoutId;
    }
    close() {
        this.isClosed = true;
    }
    flush() {
    }
    onMessage() {
    }
    getMessage() {
    }
    /**
     * Returns a map of parameters that were collected
     * during the initial http request that established the
     * connection
     *
     * @public
     * @returns {Object} handshakeData
     */
    getHandshakeData() {
        return {};
    }
    /**
     * Sends an error on the specified topic. The
     * action will automatically be set to C.ACTION.ERROR
     *
     * @param {String} topic one of C.TOPIC
     * @param {String} event one of C.EVENT
     * @param {String} msg generic error message
     *
     * @public
     * @returns {void}
     */
    sendError(message, event, errorMessage) {
        if (this.isClosed === false) {
            message_parser_1.parseData(message);
            this.onErrorCallback(this.messageResults, this.correlationIndex, message, event, errorMessage, this.responseCallback, this.requestTimeout);
        }
    }
    /**
     * Sends a message based on the provided action and topic
     *
     * @param {Object} message
     *
     * @public
     * @returns {void}
     */
    sendMessage(message) {
        if (message_parser_1.isError(message)) {
            message.isError = true;
        }
        if (this.isClosed === false) {
            message_parser_1.parseData(message);
            this.onMessageCallback(this.messageResults, this.correlationIndex, message, this.responseCallback, this.requestTimeout);
        }
    }
    sendNativeMessage(message, buffer) {
        // This can never be called as HTTP API is not a subscriber (Yet)
    }
    sendAckMessage(message) {
        message.isAck = true;
        this.sendMessage(message);
    }
    parseData(message) {
        return message_parser_1.parseData(message);
    }
    /**
     * Destroyes the socket. Removes all deepstream specific
     * logic and closes the connection
     *
     * @public
     * @returns {void}
     */
    destroy() {
    }
    onClose() {
    }
    removeOnClose() {
    }
}
exports.default = HTTPSocketWrapper;
//# sourceMappingURL=socket-wrapper.js.map