"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const sinon_1 = require("sinon");
const chai_1 = require("chai");
const message_distributor_1 = require("./message-distributor");
const testHelper = require('../test/helper/test-helper');
const test_mocks_1 = require("../test/helper/test-mocks");
const options = testHelper.getDeepstreamOptions();
const config = options.config;
const services = options.services;
describe('message connector distributes messages to callbacks', () => {
    let messageDistributor;
    let testMocks;
    let client;
    let testCallback;
    beforeEach(() => {
        testMocks = test_mocks_1.getTestMocks();
        client = testMocks.getSocketWrapper();
        testCallback = sinon_1.spy();
        messageDistributor = new message_distributor_1.default(config, services);
    });
    afterEach(() => {
        client.socketWrapperMock.verify();
    });
    it('makes remote connection', () => {
        chai_1.expect(services.message.lastSubscribedTopic).to.equal(null);
        messageDistributor.registerForTopic('someTopic', testCallback);
        chai_1.expect(services.message.lastSubscribedTopic).to.equal('someTopic');
    });
    it('makes local connection', () => {
        messageDistributor.registerForTopic('someTopic', testCallback);
        messageDistributor.distribute(client.socketWrapper, { topic: 'someTopic' });
        chai_1.expect(testCallback).to.have.callCount(1);
    });
    it.skip('routes messages from the message connector', () => {
        messageDistributor.registerForTopic('topicB', testCallback);
        services.message.simulateIncomingMessage('topicB', { topic: 'topicB' });
        chai_1.expect(testCallback).to.have.callCount(1);
    });
    it('only routes matching topics', () => {
        messageDistributor.registerForTopic('aTopic', testCallback);
        messageDistributor.registerForTopic('anotherTopic', testCallback);
        messageDistributor.distribute(client.socketWrapper, { topic: 'aTopic' });
        chai_1.expect(testCallback).to.have.callCount(1);
    });
    it('throws an error for multiple registrations to the same topic', () => {
        let hasErrored = false;
        try {
            messageDistributor.registerForTopic('someTopic', testCallback);
            messageDistributor.registerForTopic('someTopic', testCallback);
        }
        catch (e) {
            hasErrored = true;
        }
        chai_1.expect(hasErrored).to.equal(true);
    });
});
//# sourceMappingURL=message-distributor.spec.js.map