"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("mocha");
const chai_1 = require("chai");
const rulesMap = require('./rules-map');
const C = require("../constants");
describe('returns the applicable rule for a message', () => {
    it('exposes a getRulesForMessage method', () => {
        chai_1.expect(typeof rulesMap.getRulesForMessage).to.equal('function');
    });
    it('returns null for topics without rules', () => {
        const msg = {
            topic: C.TOPIC.AUTH
        };
        chai_1.expect(rulesMap.getRulesForMessage(msg)).to.equal(null);
    });
    it('returns null for actions without rules', () => {
        const msg = {
            topic: C.TOPIC.EVENT,
            action: C.EVENT_ACTIONS.UNSUBSCRIBE
        };
        chai_1.expect(rulesMap.getRulesForMessage(msg)).to.equal(null);
    });
    it('returns ruletypes for event subscribe messages', () => {
        const msg = {
            topic: C.TOPIC.EVENT,
            action: C.EVENT_ACTIONS.SUBSCRIBE
        };
        chai_1.expect(rulesMap.getRulesForMessage(msg)).to.deep.equal({
            section: 'event',
            type: 'subscribe',
            action: C.EVENT_ACTIONS.SUBSCRIBE
        });
    });
    it('returns ruletypes for record patch messages', () => {
        const msg = {
            topic: C.TOPIC.RECORD,
            action: C.RECORD_ACTIONS.PATCH
        };
        chai_1.expect(rulesMap.getRulesForMessage(msg)).to.deep.equal({
            section: 'record',
            type: 'write',
            action: C.RECORD_ACTIONS.PATCH
        });
    });
});
//# sourceMappingURL=rules-map.spec.js.map